/*
 * @(#)ImageButton.java    0.90 10/18/95 Adam Doppelt
 */

import java.awt.*;
import java.awt.image.*;

/**    
 * ImageButton is a button which supports using images to represent
 * the three states of the button - normal, pressed, and disabled. An
 * ImageButton is constructed with these three images. If null is
 * passed for <CODE>pressed</CODE> and/or <CODE>disabled</CODE>, those
 * states will draw using the <CODE>normal</CODE> image instead.<P>
 *
 * Behavior can be added to an ImageButton by subclassing it and
 * overriding the action method.<P>
 *
 * Note that the images passed to ImageButton <B>must</B> be
 * completely loaded, in order to determine what size the button
 * should be. See <A
 * HREF="http://www.cs.brown.edu/people/amd/java/GIFFactory">GIFFactory</A>
 * for a class which encapsulates this behavior.
 *
 * @see #action
 *
 * @version 0.90 18 Oct 1995
 * @author <A HREF="http://www.cs.brown.edu/people/amd/">Adam Doppelt</A> */
public class ImageButton extends Canvas {
    Image normal_, pressed_, disabled_, current_;
    boolean enabled_, down_;

/**
 * Construct an image button. An image button is composed of three
 * images, one for each possible state of the button. If null is
 * passed for pressed and/or disabled, those states will draw using the
 * normal image.
 *
 * @param normal The image to use for the button when it is enabled
 * and not being pressed by the user.
 * @param pressed The image to use for the button when it is enabled
 * and being pressed by the user.
 * @param disabled The image to use when the button is disabled.
 * */
    public ImageButton(Image normal, Image pressed, Image disabled) {
	int width = normal.getWidth(this);
	int height = normal.getHeight(this);
	if (pressed != null) {
	    width = Math.max(width, pressed.getWidth(this));
	    height = Math.max(height, pressed.getHeight(this));	    
	}
	if (disabled != null) {
	    width = Math.max(width, disabled.getWidth(this));
	    height = Math.max(height, disabled.getHeight(this));
	}
	normal_ = normal;
	pressed_ = pressed;
	disabled_ = disabled;

	current_ = normal_;
	enabled_ = true;
	down_ = false;
	
	resize(width, height);
	instant();
    }

/**
 * This method is called when the button is pressed. Override it to
 * add behavior to the button.
 *   */
    public void action() { ; }
    
/**
 * Enable the button.
 */
    public void enable() {
	enabled_ = true;
	current_ = normal_;
	instant();
    }

/**
 * Disable the button.
 */
    public void disable() {
	enabled_ = false;	
	if (disabled_ != null) {
	    current_ = disabled_;
	    instant();
	}
    }
    
/**
 * An internal method used to handle mouse down events. It should
 * never be called by the user.  */
    public boolean mouseDown(Event event, int x, int y) {
	if (enabled_) {
	    down_ = true;
	    if (pressed_ != null) {
		current_ = pressed_;
		instant();
	    }
	}
	return true;
    }

/**
 * An internal method used to handle mouse up events - it should never
 * be called by the user.  */
    public boolean mouseUp(Event event, int x, int y) {
	if (down_) {
	    down_ = false;
	    current_ = normal_;
	    instant();
	    if((x >= 0) && (y >= 0) &&
	       (x < size().width) && (y < size().height))
		action();
	}
	return true;
    }

    // updates the image on screen
    void instant() {
	Graphics onscreen = getGraphics();
	if (onscreen != null)
	    paint(onscreen);
    }
    
/**
 * This method is used to handle repaints - it should never be called
 * by the user.
 *   */
    public void paint(Graphics graphics) {
	graphics.drawImage(current_, 0, 0, this);
    }

/**
 * This method is called when additional information arrives about an
 * image. Of course, ImageButton assumes that the images have been
 * completely loaded, so this method shouldn't actually get called.<P>
 *
 * If this method <I>is</I> called, it will print out an error message
 * and call exit.
 *   */
    public boolean imageUpdate(Image image, int infoflags, int x, int y,
			       int width, int height) {
	System.out.println(
"ImageButton will _not_ work unless the image has been preloaded. Look\n" +
"at http://www.cs.brown.edu/people/amd/java/GIFFactory for a class which\n" +
"encapsulates this behavior.\n");
	System.exit(0);
	return true;
    }

}

